@@ -23,3 +23,14 @@ async def x2():
     async with open('asdf') as f:
         #? ['readlines']
         f.readlines
+
+class A():
+    @staticmethod
+    async def b(c=1, d=2):
+        return 1
+
+#! 9 ['def b']
+await A.b()
+
+#! 11 ['param d=2']
+await A.b(d=3)
