@@ -136,24 +136,6 @@ ret(1)[0]
 #? str() set()
 ret()[0]
 
-# -----------------
-# with statements
-# -----------------
-
-with open('') as f:
-    #? ['closed']
-    f.closed
-    for line in f:
-        #? str()
-        line
-
-with open('') as f1, open('') as f2:
-    #? ['closed']
-    f1.closed
-    #? ['closed']
-    f2.closed
-
-
 # -----------------
 # global vars
 # -----------------
@@ -165,6 +147,17 @@ def global_define():
 #? int()
 global_var_in_func
 
+
+def funct1():
+    # From issue #610
+    global global_dict_var
+    global_dict_var = dict()
+def funct2():
+    global global_dict_var
+    #? dict()
+    global_dict_var
+
+
 # -----------------
 # within docstrs
 # -----------------
@@ -179,6 +172,16 @@ def a():
 #?
 # str literals in comment """ upper
 
+def completion_in_comment():
+    #? ['Exception']
+    # might fail because the comment is not a leaf: Exception
+    pass
+
+some_word
+#? ['Exception']
+# Very simple comment completion: Exception
+# Commment after it
+
 # -----------------
 # magic methods
 # -----------------
@@ -215,6 +218,9 @@ if 1:
     #? str()
     xyz
 
+#?
+¹.
+
 # -----------------
 # exceptions
 # -----------------
@@ -267,3 +273,23 @@ foo
 __file__
 #? ['__file__']
 __file__
+
+
+# -----------------
+# with statements
+# -----------------
+
+with open('') as f:
+    #? ['closed']
+    f.closed
+    for line in f:
+        #? str()
+        line
+
+# Nested with statements don't exist in Python 2.6.
+# python >= 2.7
+with open('') as f1, open('') as f2:
+    #? ['closed']
+    f1.closed
+    #? ['closed']
+    f2.closed
