@@ -12,6 +12,6 @@ import pytest
 ])
 def test_equals(source):
     script = Script(source)
-    node = script._get_module_node().children[0].children[0]
+    node = script._get_module_node().children[0]
     first, = script._get_module().eval_node(node)
     assert isinstance(first, CompiledObject) and first.obj is True
